from pathlib import Path

from markdown_it.utils import read_fixture_file
import mdformat
import mdformat._cli
import pytest

TEST_CASES = read_fixture_file(Path(__file__).parent / "data" / "fixtures.md")

print([f[1] for f in TEST_CASES])


@pytest.mark.parametrize(
    "line,title,text,expected", TEST_CASES, ids=[f[1] for f in TEST_CASES]
)
def test_fixtures__api(line, title, text, expected):
    """Test fixtures in tests/data/fixtures.md."""
    md_new = mdformat.text(text, extensions={"myst"})
    try:
        assert md_new == expected
    except Exception:
        print(md_new)
        raise


@pytest.mark.parametrize(
    "line,title,text,expected", TEST_CASES, ids=[f[1] for f in TEST_CASES]
)
def test_fixtures__cli(line, title, text, expected, tmp_path):
    """Test fixtures in tests/data/fixtures.md."""
    file_path = tmp_path / "test_markdown.md"
    file_path.write_text(text)
    assert mdformat._cli.run([str(file_path)]) == 0
    md_new = file_path.read_text()
    assert md_new == expected



import subprocess
import sys

from pathlib import Path
from typing import Optional, List


def run_formatter(
    path: Path or str or None = None,
    check: bool = False,
    version: bool = False,
    verbose: bool = False,
    exclude: Optional[List[str]] = None,
    subprocess_check: bool = True,
) -> subprocess.CompletedProcess:
    """
    Run the mdformat-py-edu-fr formatter on a path.

    Args:
        path: Path to file or directory to format
        check: If True, run in check mode (--check flag)

    Returns:
        CompletedProcess instance
    """
    cmd = [sys.executable, "-m", "mdformat_py_edu_fr"]
    if check:
        cmd.append("--check")

    if version:
        cmd.append("--version")

    if verbose:
        cmd.append("-v")

    if path is not None:
        cmd.append(str(path))

    if exclude:
        for pattern in exclude:
            cmd.extend(["--exclude", pattern])

    return subprocess.run(
        cmd,
        capture_output=True,
        text=True,
        check=subprocess_check,
    )



from pathlib import Path

from .conftest import run_formatter


def test_mdformat_error():
    path_mdformat_error = Path(__file__).parent / "examples/mdformat_error.md"
    assert path_mdformat_error.exists()

    process = run_formatter(path_mdformat_error, subprocess_check=False)
    assert process.returncode
    assert process.stderr.startswith("Error: mdformat exception for file")
    assert "examples/mdformat_error.md" in process.stderr



"""Test command-line interface behavior."""

from .conftest import run_formatter


def test_version_flag():
    """Test that --version works."""

    result = run_formatter(version=True)

    assert result.returncode == 0
    assert "mdformat-py-edu-fr" in result.stdout


def test_nonexistent_file():
    """Test behavior with non-existent file."""
    result = run_formatter("/nonexistent/file.md", subprocess_check=False)

    # Should fail gracefully
    assert result.returncode != 0


def test_directory_processing(tmp_path):
    """Test that directories are processed recursively."""
    # Create nested structure
    (tmp_path / "subdir").mkdir()
    (tmp_path / "test1.md").write_text("# Test 1\n", encoding="utf-8")
    (tmp_path / "subdir" / "test2.md").write_text("# Test 2\n", encoding="utf-8")

    result = run_formatter(tmp_path)

    assert result.returncode == 0



"""Test the --exclude flag behavior."""

import pytest

from .conftest import run_formatter


def test_exclude_single_pattern(tmp_path):
    """Test excluding files with a single pattern."""
    # Create structure
    (tmp_path / "include.md").write_text("# Include\n", encoding="utf-8")
    (tmp_path / "exclude.md").write_text("# Exclude\n", encoding="utf-8")
    (tmp_path / "subdir").mkdir()
    (tmp_path / "subdir" / "nested.md").write_text("# Nested\n", encoding="utf-8")

    # Run with exclude pattern
    result = run_formatter(tmp_path, exclude=["exclude.md"], verbose=True)

    # Check that exclude.md was not mentioned in verbose output
    assert "exclude.md" not in result.stdout
    # But include.md should be processed
    assert "include.md" in result.stdout or result.returncode == 0


def test_exclude_multiple_patterns(tmp_path):
    """Test excluding files with multiple patterns."""
    # Create structure
    (tmp_path / "keep.md").write_text("# Keep\n", encoding="utf-8")
    (tmp_path / "draft_1.md").write_text("# Draft 1\n", encoding="utf-8")
    (tmp_path / "draft_2.md").write_text("# Draft 2\n", encoding="utf-8")
    (tmp_path / "_build/html").mkdir(parents=True)
    (tmp_path / "_build/html" / "generated.md").write_text(
        "# Generated\n", encoding="utf-8"
    )

    # Run with multiple exclude patterns
    result = run_formatter(
        tmp_path, exclude=["draft_*.md", "_build/**/*.md"], verbose=True
    )

    # Check that excluded files were not mentioned
    assert "draft_1.md" not in result.stdout
    assert "draft_2.md" not in result.stdout
    assert "generated.md" not in result.stdout


def test_exclude_directory_pattern(tmp_path):
    """Test excluding entire directories."""
    # Create structure with .ipynb_checkpoints
    (tmp_path / "main.md").write_text("# Main\n", encoding="utf-8")
    (tmp_path / ".ipynb_checkpoints").mkdir()
    (tmp_path / ".ipynb_checkpoints" / "checkpoint.md").write_text(
        "# Checkpoint\n", encoding="utf-8"
    )
    (tmp_path / "subdir").mkdir()
    (tmp_path / "subdir" / ".ipynb_checkpoints").mkdir()
    (tmp_path / "subdir" / ".ipynb_checkpoints" / "nested.md").write_text(
        "# Nested\n", encoding="utf-8"
    )
    (tmp_path / "subdir" / "real.md").write_text("# Real\n", encoding="utf-8")

    # Exclude all .ipynb_checkpoints directories
    result = run_formatter(tmp_path, exclude=["**/.ipynb_checkpoints/*"], verbose=True)

    # Checkpoints should not be processed
    assert "checkpoint.md" not in result.stdout
    assert ".ipynb_checkpoints" not in result.stdout
    # But real files should be
    assert "main.md" in result.stdout or result.returncode == 0


def test_exclude_with_check_mode(tmp_path):
    """Test that --exclude works with --check mode."""
    # Create files
    (tmp_path / "format_me.md").write_text("# Format Me\n", encoding="utf-8")
    (tmp_path / "skip_me.md").write_text("# Skip Me\n", encoding="utf-8")

    # Run with both --check and --exclude
    result = run_formatter(tmp_path, check=True, exclude=["skip_me.md"], verbose=True)

    # Should only check format_me.md
    # skip_me.md should not appear in output
    if "skip_me.md" in result.stderr or "skip_me.md" in result.stdout:
        pytest.fail("Excluded file was processed in check mode")



import subprocess
import sys


class RuffFormattingError(Exception):
    """Exception raised when ruff formatting fails."""

    pass


def format_code_with_ruff(code: str) -> str:
    """Format Python code using ruff.

    Returns formatted code.

    Raises:
        RuffFormattingError: If ruff formatting fails.
    """
    result = subprocess.run(
        ["ruff", "format", "-"],
        input=code.encode("utf-8"),
        capture_output=True,
        check=False,
    )

    if result.returncode != 0:
        error_msg = result.stderr.decode("utf-8")
        raise RuffFormattingError(error_msg)

    return result.stdout.decode("utf-8")



"""
Tests for mdformat-py-edu-fr formatter.

Test structure:
- tests/examples/unformatted/ - Contains unformatted Markdown files
- tests/examples/formatted/ - Contains the expected formatted output
"""

import pytest
from pathlib import Path
from typing import List

from .conftest import run_formatter

from mdformat_py_edu_fr import format_file

EXAMPLES_DIR = Path(__file__).parent / "examples"
UNFORMATTED_DIR = EXAMPLES_DIR / "unformatted"
FORMATTED_DIR = EXAMPLES_DIR / "formatted"


def get_test_files() -> List[tuple[Path, Path]]:
    """
    Get pairs of (unformatted, formatted) test files.

    Returns:
        List of tuples (unformatted_path, formatted_path)
    """
    if not UNFORMATTED_DIR.exists() or not FORMATTED_DIR.exists():
        return []

    pairs = []
    for unformatted_file in UNFORMATTED_DIR.rglob("*.md"):
        # Get relative path from unformatted dir
        rel_path = unformatted_file.relative_to(UNFORMATTED_DIR)
        formatted_file = FORMATTED_DIR / rel_path

        if formatted_file.exists():
            pairs.append((unformatted_file, formatted_file))
        else:
            pytest.fail(f"Missing formatted version for: {rel_path}")

    return pairs


@pytest.fixture
def temp_file(tmp_path):
    """Create a temporary file for testing."""

    def _create_temp_file(file_path: Path) -> tuple[Path, str]:
        content = file_path.read_text(encoding="utf-8")
        temp = tmp_path / file_path.name
        temp.write_text(content, encoding="utf-8")
        return temp, content

    return _create_temp_file


class TestFormatting:
    """Test that unformatted files are correctly formatted."""

    @pytest.mark.parametrize("unformatted,formatted", get_test_files())
    def test_format_matches_expected(
        self, unformatted: Path, formatted: Path, temp_file
    ):
        """Test that formatting produces the expected output."""
        expected_content = formatted.read_text(encoding="utf-8")

        temp, _ = temp_file(unformatted)

        actual_content = format_file(temp)

        # Read the formatted output
        actual_content = temp.read_text(encoding="utf-8")

        # Compare with expected
        assert actual_content == expected_content, (
            f"Formatted output doesn't match expected for {unformatted.name}"
        )


class TestStability:
    """Test that formatting is stable (idempotent)."""

    @pytest.mark.parametrize("unformatted,formatted", get_test_files())
    def test_format_is_stable(self, formatted: Path, temp_file, unformatted: Path):
        """Test that formatting already-formatted files doesn't change them."""
        # Create a temporary file with formatted content
        temp, formatted_content = temp_file(formatted)

        # Run the formatter on already-formatted content
        result = run_formatter(temp)

        assert result.returncode == 0, f"Formatter failed: {result.stderr}"

        # Read the output
        output_content = temp.read_text(encoding="utf-8")

        # Should be identical (stable/idempotent)
        assert output_content == formatted_content, (
            f"Formatter is not stable for {formatted.name}. "
            "Running it twice produces different results."
        )

    @pytest.mark.parametrize("unformatted,formatted", get_test_files())
    def test_double_format_is_stable(
        self, unformatted: Path, temp_file, formatted: Path
    ):
        """Test that formatting twice produces the same result."""
        temp, _ = temp_file(unformatted)

        # Format once
        result1 = run_formatter(temp)
        assert result1.returncode == 0, f"First format failed: {result1.stderr}"

        first_format = temp.read_text(encoding="utf-8")

        # Format again
        result2 = run_formatter(temp)

        assert result2.returncode == 0, f"Second format failed: {result2.stderr}"

        second_format = temp.read_text(encoding="utf-8")

        # Both formats should be identical
        assert first_format == second_format, (
            f"Formatting {unformatted.name} twice produces different results"
        )


class TestCheckMode:
    """Test the --check flag behavior."""

    def test_check_unformatted_fails(self, temp_file):
        """Test that --check returns non-zero for unformatted files."""
        # Get first unformatted file
        test_files = get_test_files()
        if not test_files:
            pytest.skip("No test files found")

        unformatted, _ = test_files[0]
        temp, _ = temp_file(unformatted)

        result = run_formatter(temp, check=True, subprocess_check=False)

        # Should return non-zero (file needs formatting)
        assert result.returncode != 0

    def test_check_formatted_passes(self, temp_file):
        """Test that --check returns zero for already-formatted files."""
        # Get first formatted file
        test_files = get_test_files()
        if not test_files:
            pytest.skip("No test files found")

        _, formatted = test_files[0]
        temp, _ = temp_file(formatted)

        result = run_formatter(temp, check=True)

        # Should return zero (file is already formatted)
        assert result.returncode == 0

    def test_check_doesnt_modify_file(self, temp_file):
        """Test that --check doesn't modify files."""
        # Get first unformatted file
        test_files = get_test_files()
        if not test_files:
            pytest.skip("No test files found")

        unformatted, _ = test_files[0]
        temp, original_content = temp_file(unformatted)

        # Run with --check
        run_formatter(temp, check=True, subprocess_check=False)

        # File should be unchanged
        final_content = temp.read_text(encoding="utf-8")
        assert final_content == original_content



import pytest

from pathlib import Path

from markdown_it.utils import read_fixture_file

from mdformat_py_edu_fr.util_ruff import format_code_with_ruff, RuffFormattingError


TEST_CASES = read_fixture_file(Path(__file__).parent / "./fixtures_ruff.txt")


@pytest.mark.parametrize(
    "line,title,text,expected", TEST_CASES, ids=[f[1] for f in TEST_CASES]
)
def test_format_with_ruff(line, title, text, expected):
    assert format_code_with_ruff(text) == expected


def test_error():
    code = "a ==== 1"

    with pytest.raises(RuffFormattingError, match="error: Failed to parse"):
        format_code_with_ruff(code)



import json
from pathlib import Path

import mdformat
from mdformat._util import is_md_equal
import pytest

SPECTESTS_PATH = Path(__file__).parent / "data" / "commonmark_spec_v0.29.json"
SPECTESTS_CASES = tuple(
    {"name": str(entry["example"]), "md": entry["markdown"]}
    for entry in json.loads(SPECTESTS_PATH.read_text(encoding="utf-8"))
)


@pytest.mark.parametrize(
    "entry", SPECTESTS_CASES, ids=[c["name"] for c in SPECTESTS_CASES]
)
def test_commonmark_spec(entry):
    """Test mdformat-myst against the CommonMark spec.

    Test that:
    1. Markdown AST is the same before and after 1 pass of formatting
    2. Markdown after 1st pass and 2nd pass of formatting are equal
    """
    md_original = entry["md"]
    md_new = mdformat.text(md_original, extensions={"myst"})
    md_2nd_pass = mdformat.text(md_new, extensions={"myst"})
    assert is_md_equal(md_original, md_new, extensions={"myst"})
    assert md_new == md_2nd_pass



from __future__ import annotations

from collections.abc import Mapping, MutableMapping, Sequence
import io

from markdown_it import MarkdownIt
from mdformat.renderer import LOGGER, RenderContext, RenderTreeNode
import ruamel.yaml

yaml = ruamel.yaml.YAML()
yaml.indent(mapping=2, sequence=4, offset=2)


def longest_consecutive_sequence(seq: str, char: str) -> int:
    """Return length of the longest consecutive sequence of `char` characters
    in string `seq`."""
    assert len(char) == 1
    longest = 0
    current_streak = 0
    for c in seq:
        if c == char:
            current_streak += 1
        else:
            current_streak = 0
        if current_streak > longest:
            longest = current_streak
    return longest


def fence(node: "RenderTreeNode", context: "RenderContext") -> str:
    """Render fences (and directives).

    Copied from upstream `mdformat` core and should be kept up-to-date
    if upstream introduces changes. Note that only two lines are added
    to the upstream implementation, i.e. the condition that calls
    `format_directive_content` function.
    """
    info_str = node.info.strip()
    lang = info_str.split(maxsplit=1)[0] if info_str else ""
    code_block = node.content

    # Info strings of backtick code fences can not contain backticks or tildes.
    # If that is the case, we make a tilde code fence instead.
    if "`" in info_str or "~" in info_str:
        fence_char = "~"
    else:
        fence_char = "`"

    # Format the code block using enabled codeformatter funcs
    if lang in context.options.get("codeformatters", {}):
        fmt_func = context.options["codeformatters"][lang]
        try:
            code_block = fmt_func(code_block, info_str)
        except Exception:
            # Swallow exceptions so that formatter errors (e.g. due to
            # invalid code) do not crash mdformat.
            assert node.map is not None, "A fence token must have `map` attribute set"
            LOGGER.warning(
                f"Failed formatting content of a {lang} code block "
                f"(line {node.map[0] + 1} before formatting)"
            )
    # This "elif" is the *only* thing added to the upstream `fence` implementation!
    elif lang.startswith("{") and lang.endswith("}"):
        code_block = format_directive_content(code_block)

    # The code block must not include as long or longer sequence of `fence_char`s
    # as the fence string itself
    fence_len = max(3, longest_consecutive_sequence(code_block, fence_char) + 1)
    fence_str = fence_char * fence_len

    return f"{fence_str}{info_str}\n{code_block}{fence_str}"


def format_directive_content(raw_content: str) -> str:
    parse_result = parse_opts_and_content(raw_content)
    if not parse_result:
        return raw_content
    unformatted_yaml, content = parse_result
    dump_stream = io.StringIO()
    try:
        parsed = yaml.load(unformatted_yaml)
        yaml.dump(parsed, stream=dump_stream)
    except ruamel.yaml.YAMLError:
        LOGGER.warning("Invalid YAML in MyST directive options.")
        return raw_content
    formatted_yaml = dump_stream.getvalue()

    # Remove the YAML closing tag if added by `ruamel.yaml`
    if formatted_yaml.endswith("\n...\n"):
        formatted_yaml = formatted_yaml[:-4]

    # Convert empty YAML to most concise form
    if formatted_yaml == "null\n":
        formatted_yaml = ""

    formatted = "---\n" + formatted_yaml + "---\n"
    if content:
        formatted += content + "\n"
    return formatted


def parse_opts_and_content(raw_content: str) -> tuple[str, str] | None:
    if not raw_content:
        return None
    lines = raw_content.splitlines()
    line = lines.pop(0)
    yaml_lines = []
    if all(c == "-" for c in line) and len(line) >= 3:
        while lines:
            line = lines.pop(0)
            if all(c == "-" for c in line) and len(line) >= 3:
                break
            yaml_lines.append(line)
    elif line.lstrip().startswith(":"):
        yaml_lines.append(line.lstrip()[1:])
        while lines:
            if not lines[0].lstrip().startswith(":"):
                break
            line = lines.pop(0).lstrip()[1:]
            yaml_lines.append(line)
    else:
        return None

    first_line_is_empty_but_second_line_isnt = (
        len(lines) >= 2 and not lines[0].strip() and lines[1].strip()
    )
    exactly_one_empty_line = len(lines) == 1 and not lines[0].strip()
    if first_line_is_empty_but_second_line_isnt or exactly_one_empty_line:
        lines.pop(0)

    unformatted_yaml = "\n".join(yaml_lines)
    content = "\n".join(lines)
    return unformatted_yaml, content


def render_fence_html(
    self: MarkdownIt, tokens: Sequence, idx: int, options: Mapping, env: MutableMapping
) -> str:
    return ""



from pathlib import Path

from .conftest import run_formatter


def test_ruff_error():
    path_ruff_error = Path(__file__).parent / "examples/ruff_error.md"
    assert path_ruff_error.exists()

    process = run_formatter(path_ruff_error, subprocess_check=False)
    assert process.returncode
    assert process.stderr.startswith("Ruff formatting failed for file")



from __future__ import annotations

import re
import textwrap

from markdown_it import MarkdownIt
import mdformat.plugins
from mdformat.renderer import RenderContext, RenderTreeNode
from mdit_py_plugins.dollarmath import dollarmath_plugin
from mdit_py_plugins.myst_blocks import myst_block_plugin
from mdit_py_plugins.myst_role import myst_role_plugin
from mdit_py_plugins.container import container_plugin

from mdformat_myst_pef._directives import fence, render_fence_html

_TARGET_PATTERN = re.compile(r"^\s*\(.+\)=\s*$")
_ROLE_NAME_PATTERN = re.compile(r"({[a-zA-Z0-9_\-+:]+})")
_YAML_HEADER_PATTERN = re.compile(r"(?m)(^:\w+: .*$\n?)+|^---$\n(?s:.).*\n---\n")

container_names = [
    "admonition",
    "attention",
    "caution",
    "danger",
    "div",
    "dropdown",
    "embed",
    "error",
    "exercise",
    "exercise-end",
    "exercise-start",
    "figure",
    "glossary",
    "grid",
    "grid-item",
    "grid-item-card",
    "hint",
    "image",
    "important",
    "include",
    "index",
    "literal-include",
    "margin",
    "math",
    "note",
    "prf:algorithm",
    "prf:assumption",
    "prf:axiom",
    "prf:conjecture",
    "prf:corollary",
    "prf:criterion",
    "prf:definition",
    "prf:example",
    "prf:lemma",
    "prf:observation",
    "prf:proof",
    "prf:property",
    "prf:proposition",
    "prf:remark",
    "prf:theorem",
    "seealso",
    "show-index",
    "sidebar",
    "solution",
    "solution-end",
    "solution-start",
    "span",
    "tab-item",
    "tab-set",
    "table",
    "tip",
    "todo",
    "topics",
    "warning",
]


def update_mdit(mdit: MarkdownIt) -> None:
    # Enable mdformat-tables plugin
    tables_plugin = mdformat.plugins.PARSER_EXTENSIONS["tables"]
    if tables_plugin not in mdit.options["parser_extension"]:
        mdit.options["parser_extension"].append(tables_plugin)
        tables_plugin.update_mdit(mdit)

    # Enable mdformat-frontmatter plugin
    frontmatter_plugin = mdformat.plugins.PARSER_EXTENSIONS["frontmatter"]
    if frontmatter_plugin not in mdit.options["parser_extension"]:
        mdit.options["parser_extension"].append(frontmatter_plugin)
        frontmatter_plugin.update_mdit(mdit)

    # Enable mdformat-footnote plugin
    footnote_plugin = mdformat.plugins.PARSER_EXTENSIONS["footnote"]
    if footnote_plugin not in mdit.options["parser_extension"]:
        mdit.options["parser_extension"].append(footnote_plugin)
        footnote_plugin.update_mdit(mdit)

    # Enable MyST role markdown-it extension
    mdit.use(myst_role_plugin)

    # Enable MyST block markdown-it extension (including "LineComment",
    # "BlockBreak" and "Target" syntaxes)
    mdit.use(myst_block_plugin)

    # Enable dollarmath markdown-it extension
    mdit.use(dollarmath_plugin)

    # Trick `mdformat`s AST validation by removing HTML rendering of code
    # blocks and fences. Directives are parsed as code fences and we
    # modify them in ways that don't break MyST AST but do break
    # CommonMark AST, so we need to do this to make validation pass.
    mdit.add_render_rule("fence", render_fence_html)
    mdit.add_render_rule("code_block", render_fence_html)

    for name in container_names:
        container_plugin(mdit, name="{" + name + "}", marker=":")


def container_renderer(
    node: RenderTreeNode, context: RenderContext, *args, **kwargs
) -> str:
    children = node.children
    paragraphs = []
    if children:
        # Look at the tokens forming the first paragraph and see if
        # they form a YAML header. This could be stricter: there
        # should be exactly three tokens: paragraph open, YAML
        # header, paragraph end.
        tokens = children[0].to_tokens()
        if all(
            token.type in {"paragraph_open", "paragraph_close"}
            or _YAML_HEADER_PATTERN.fullmatch(token.content)
            for token in tokens
        ):
            paragraphs.append(
                "\n".join(token.content.strip() for token in tokens if token.content)
            )
            # and skip that first paragraph
            children = children[1:]

    paragraphs.extend(child.render(context) for child in children)

    return node.markup + node.info + "\n" + "\n\n".join(paragraphs) + "\n" + node.markup


def _role_renderer(node: RenderTreeNode, context: RenderContext) -> str:
    role_name = "{" + node.meta["name"] + "}"
    role_content = f"`{node.content}`"
    return role_name + role_content


def _comment_renderer(node: RenderTreeNode, context: RenderContext) -> str:
    return "%" + node.content.replace("\n", "\n%")


def _blockbreak_renderer(node: RenderTreeNode, context: RenderContext) -> str:
    text = "+++"
    if node.content:
        text += f" {node.content}"
    return text


def _target_renderer(node: RenderTreeNode, context: RenderContext) -> str:
    return f"({node.content})="


def _math_inline_renderer(node: RenderTreeNode, context: RenderContext) -> str:
    return f"${node.content}$"


def _math_block_renderer(node: RenderTreeNode, context: RenderContext) -> str:
    indent_width = context.env.get("indent_width", 0)
    if indent_width > 0:
        return f"$${textwrap.dedent(node.content)}$$"
    return f"$${node.content}$$"


def _math_block_label_renderer(node: RenderTreeNode, context: RenderContext) -> str:
    return f"{_math_block_renderer(node, context)} ({node.info})"


def _math_block_safe_blockquote_renderer(
    node: RenderTreeNode, context: RenderContext
) -> str:
    marker = "> "
    with context.indented(len(marker)):
        lines = []
        for i, child in enumerate(node.children):
            if child.type in ("math_block", "math_block_label"):
                lines.append(child.render(context))
            else:
                lines.extend(child.render(context).splitlines())
            if i < (len(node.children) - 1):
                lines.append("")
        if not lines:
            return ">"
        quoted_lines = (f"{marker}{line}" if line else ">" for line in lines)
        quoted_str = "\n".join(quoted_lines)
        return quoted_str


def _render_children(node: RenderTreeNode, context: RenderContext) -> str:
    return "\n\n".join(child.render(context) for child in node.children)


def _escape_paragraph(text: str, node: RenderTreeNode, context: RenderContext) -> str:
    lines = text.split("\n")

    for i in range(len(lines)):
        # Three or more "+" chars are interpreted as a block break. Escape them.
        space_removed = lines[i].replace(" ", "")
        if space_removed.startswith("+++"):
            lines[i] = lines[i].replace("+", "\\+", 1)

        # A line starting with "%" is a comment. Escape.
        if lines[i].startswith("%"):
            lines[i] = f"\\{lines[i]}"

        # Escape lines that look like targets
        if _TARGET_PATTERN.search(lines[i]):
            lines[i] = lines[i].replace("(", "\\(", 1)

    return "\n".join(lines)


def _escape_text(text: str, node: RenderTreeNode, context: RenderContext) -> str:
    # Escape MyST role names
    text = _ROLE_NAME_PATTERN.sub(r"\\\1", text)

    # Escape inline and block dollarmath
    text = text.replace("$", "\\$")

    return text


RENDERERS = {
    "blockquote": _math_block_safe_blockquote_renderer,
    "myst_role": _role_renderer,
    "myst_line_comment": _comment_renderer,
    "myst_block_break": _blockbreak_renderer,
    "myst_target": _target_renderer,
    "math_inline": _math_inline_renderer,
    "math_block_label": _math_block_label_renderer,
    "math_block": _math_block_renderer,
    "fence": fence,
}


for name in container_names:
    RENDERERS["container_{" + name + "}"] = container_renderer

POSTPROCESSORS = {"paragraph": _escape_paragraph, "text": _escape_text}



import sys
from pathlib import Path
from typing import List

import mdformat

from .format_with_jupytext import format_md_with_jupytext

try:
    from importlib.metadata import version

    __version__ = version("mdformat-py-edu-fr")
except Exception:
    __version__ = "unknown"

from mdformat._conf import DEFAULT_OPTS

options = DEFAULT_OPTS.copy()
options.update({"number": True, "wrap": 89, "end_of_line": "lf"})


def _match_patterns(path: Path, patterns: List[str]) -> bool:
    """
    Check if a path matches any of the given glob patterns.

    Args:
        path: Path to check
        patterns: List of glob patterns

    Returns:
        True if path matches any pattern, False otherwise
    """
    for pattern in patterns:
        if path.match(pattern) or path.full_match(pattern):
            return True
    return False


def collect_markdown_files(
    paths: List[Path], exclude_patterns: List[str] = None
) -> List[Path]:
    """
    Collect all Markdown files from the given paths.

    Args:
        paths: List of file or directory paths
        exclude_patterns: List of glob patterns to exclude

    Returns:
        List of Path objects pointing to Markdown files
    """
    if exclude_patterns is None:
        exclude_patterns = []

    markdown_files = []

    for path in paths:
        if not path.exists():
            print(f"Error: Path does not exist: {path}", file=sys.stderr)
            continue

        if path.is_file():
            if path.suffix in {".md", ".markdown"}:
                # Check if file matches any exclude pattern
                if not _match_patterns(path, exclude_patterns):
                    markdown_files.append(path)
            else:
                print(f"Warning: Skipping non-Markdown file: {path}", file=sys.stderr)
        elif path.is_dir():
            # Recursively find all Markdown files
            for md_file in path.rglob("*.md"):
                if not _match_patterns(md_file, exclude_patterns):
                    markdown_files.append(md_file)
            for md_file in path.rglob("*.markdown"):
                if not _match_patterns(md_file, exclude_patterns):
                    markdown_files.append(md_file)
        else:
            print(f"Warning: Skipping special file: {path}", file=sys.stderr)

    return sorted(set(markdown_files))


def format_file(filepath: Path, check: bool = False, verbose: bool = False) -> bool:
    """
    Format a single Markdown file.

    Args:
        filepath: Path to the file to format
        check: If True, only check formatting without modifying
        verbose: If True, print detailed information

    Returns:
        True if file is properly formatted (or was formatted), False otherwise
    """
    if verbose:
        print(f"Processing: {filepath}")

    original_str = filepath.read_text()

    enabled_parserplugins = mdformat.plugins.PARSER_EXTENSIONS

    try:
        formatted_str = mdformat.text(
            original_str,
            options=options,
            extensions=enabled_parserplugins,
            _filename=str(filepath),
        )
    except Exception as exc:
        print(f"Error: mdformat exception for file {filepath}", file=sys.stderr)
        raise exc

    formatted_str = format_md_with_jupytext(formatted_str, filepath)

    formatted = formatted_str == original_str

    if check or formatted:
        return formatted

    filepath.write_text(formatted_str)
    print(f"{filepath} reformatted")
    return formatted



"""
Markdown formatter for py-edu-fr with mdformat, mdformat-myst, and jupytext support.
"""

import argparse
import sys

from pathlib import Path

from . import __version__, format_file, collect_markdown_files


def get_parser() -> argparse.ArgumentParser:
    """Create and configure the argument parser."""
    parser = argparse.ArgumentParser(
        prog="mdformat-py-edu-fr",
        description="Format Markdown files for py-edu-fr project",
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )

    parser.add_argument(
        "--version",
        action="version",
        version=f"%(prog)s {__version__}",
    )

    parser.add_argument(
        "paths",
        nargs="*",
        type=Path,
        help="Files or directories to format (if omitted, reads from stdin)",
    )

    parser.add_argument(
        "--check",
        action="store_true",
        help="Check if files are formatted without modifying them (exit code 1 if changes needed)",
    )

    parser.add_argument(
        "--exclude",
        action="append",
        default=["**/_build/**", "**/.ipynb_checkpoints/**", "**/.venv/**"],
        metavar="PATTERN",
        help="Glob pattern to exclude files/directories (can be specified multiple times)",
    )

    parser.add_argument(
        "--verbose",
        "-v",
        action="store_true",
        help="Print detailed information about processing",
    )

    parser.add_argument(
        "--quiet",
        "-q",
        action="store_true",
        help="Disable printing information",
    )

    return parser


def main() -> int:
    """
    Main entrypoint for the mdformat-py-edu-fr command.

    Returns:
        Exit code (0 for success, 1 for failure)
    """
    parser = get_parser()
    args = parser.parse_args()

    # Handle stdin if no paths provided
    if not args.paths:
        print("Error: Reading from stdin not yet implemented", file=sys.stderr)
        return 1

    # Collect all Markdown files
    markdown_files = collect_markdown_files(args.paths, args.exclude)

    if not markdown_files:
        print("No Markdown files found", file=sys.stderr)
        return 1

    if not args.quiet and not all(path.name.endswith(".md") for path in args.paths):
        print(f"Processing {len(markdown_files)} Markdown file(s)")

    # Process each file
    all_formatted = True
    for filepath in markdown_files:
        formatted = format_file(filepath, check=args.check, verbose=args.verbose)
        if not formatted:
            all_formatted = False
            if args.check:
                print(f"Would reformat: {filepath}")

    # Return appropriate exit code
    if args.check and not all_formatted:
        return 1

    return 0


if __name__ == "__main__":
    sys.exit(main())



import jupytext
import re
import sys

from pathlib import Path

from mdformat_py_edu_fr.util_ruff import format_code_with_ruff, RuffFormattingError


LABEL_PATTERN = re.compile("(?m)(^\\s*\\(\\w+\\)=)(\r?\n)(\\s*\r?\n)")

path_config_file = Path(__file__).absolute().parent / "jupytext.toml"
config = jupytext.config.load_jupytext_configuration_file(str(path_config_file))


def format_md_with_jupytext(text: str, path_input_file: Path | None = None) -> str:
    """Format the code of a notebook code using jupytext"""

    notebook = jupytext.reads(text, fmt="md:myst")
    if "kernelspec" not in notebook.metadata:
        return text
    language = notebook["metadata"]["kernelspec"]["language"]
    if "learning" in notebook.metadata:
        # Fix common errors in learning metadata
        learning = notebook.metadata["learning"]
        for a in ["prerequisites", "objectives"]:
            # Fix singular instead of plural
            singular = a[:-1]
            if singular in learning:
                learning[a] = learning[singular]
                del learning[singular]

            # Fix string instead of list of string
            for b in ["discover", "remember", "understand", "apply"]:
                if a in learning and isinstance(learning[a].get(b), str):
                    learning[a][b] = [s.strip() for s in learning[a][b].split(",")]

    for cell in notebook.cells:
        if (
            language == "python"
            and cell["cell_type"] == "code"
            and cell["metadata"].get("format", True)
            and not all(
                any(line.strip().startswith(char) for char in "#%!")
                for line in cell.source.splitlines()
            )
        ):
            try:
                cell.source = format_code_with_ruff(cell["source"].strip())
            except RuffFormattingError as exc:
                print(
                    f"Ruff formatting failed for file\n{path_input_file or '???'}\n"
                    f"cell source:\n{cell.source}\n",
                    exc,
                    file=sys.stderr,
                )
                sys.exit(1)

        if (
            cell.metadata is not None
            and "tags" in cell.metadata
            and cell.metadata["tags"] == []
        ):
            del cell.metadata["tags"]
        # Workaround to remove empty lines between label and subsequent item
        if cell.cell_type == "markdown":
            cell.source = re.sub(LABEL_PATTERN, r"\1\2", cell.source)

    result = jupytext.writes(notebook, fmt="md:myst", config=config)
    if language == "python":
        result = result.replace("```{code-cell} ipython3", "```{code-cell}")
    return result



import re

from mdformat_py_edu_fr.format_with_jupytext import LABEL_PATTERN


def test_label_pattern():
    assert (
        re.sub(
            LABEL_PATTERN,
            r"\1\2",
            """
                  (label)=


                  bla
                  """,
        )
        == """
                  (label)=
                  bla
                  """
    )
    assert (
        re.sub(
            LABEL_PATTERN,
            r"\1\2",
            """
                  (oper)=

                  :::{admonition} Opérations arithmétiques sur les entiers
                  """,
        )
        == """
                  (oper)=
                  :::{admonition} Opérations arithmétiques sur les entiers
                  """
    )
